<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 */
class ShortUrlModule extends CWebModule
{

    /**
     * @var int The conversion base
     */
    public $conversionBase = 36;

    /**
     * @var array Map of supported object types 
     */
    public $objectTypeMap = array(
        'p' => 10007,
        'f' => 10008
    );



    /**
     * Transform a short object type and ID to a content ID
     * @param type $type Base encoded object type
     * @param type $encodedId Base encoded object Id
     * @return type A DCR content ID or null on error
     */
    public function transformToContentId($type, $encodedId)
    {
        return
            $this->shortTypeToObjectType($type)
            . "."
            . base_convert($encodedId, $this->conversionBase, 10);
    }

    /**
     * Base encode a content ID
     * @param string $contentId The content ID of a CMS object (including its type). eg.: 10007.123
     */
    public function transformToBaseEncoded($contentId)
    {
        $contentIdParts = explode(".", $contentId, 2);
        if (sizeof($contentIdParts) !== 2) {
            return null;
        }

        $objectType = $contentIdParts[0];
        $objectId   = $contentIdParts[1];

        $shortType = array_search($objectType, $this->objectTypeMap);
        if (!$shortType) {
            return null;
        }
        
        return $shortType . base_convert($objectId, 10, $this->conversionBase);
    }

    /**
     * Convert a short type to a CMS object type ID
     * @param int $type A single letter representation of the type. e.g.: 'p'
     * @return int A CMS object type or null if $type was not found
     */
    public function shortTypeToObjectType($type)
    {
        return isset($this->objectTypeMap[$type]) ? $this->objectTypeMap[$type] : null;
    }

    /**
     * Convert a CMS object type ID to a single letter representation
     * @param int $objectType An CMS object type like 10007. e.g.: 10007
     * @return string A letter like 'p' or null if unsupported
     */
    public function objectTypeToShortType($objectType)
    {
        $shortType = array_search($objectType, $this->objectTypeMap);

        return $shortType ? $shortType : null;
    }

    /**
     * Create a short URL
     * @param string $contentId The content ID
     * @param array $params Additional get parameters
     * @return string The generated URL
     * @throws CHttpException When the content ID coukld not be transformed
     */
    public function createUrl($contentId, $params = array())
    {
        $encoded = Yii::app()->getModule('shorturl')->transformToBaseEncoded($contentId);

        if ($encoded == null) {
            throw new CHttpException(500, "Could not base transform the content ID " . $contentId);
        }

        $shortType = substr($encoded, 0, 1);
        $encodedId = substr($encoded, 1);

        return Yii::app()->createurl('s/' . $shortType . $encodedId, $params); 
    }
}
